require_relative "aspect"

class Job
  def run
    raise "subclass must override Command#execute"
  end
end

class PrintJob < Job
  def run
    puts "running PrintJob..."
  end
end

class BackupJob < Job
  def run
    puts "running BackupJob..."
  end
end

module Logging
  extend Aspect

  RUN = execution(class: PrintJob, method: :run)

  before RUN do |join_point|
    STDERR.puts("before #{join_point.signature}")
  end

  after RUN do |join_point|
    STDERR.puts("after #{join_point.signature}")
  end

  around RUN do |join_point|
    STDERR.puts("enter #{join_point.signature}")
    begin
      join_point.proceed
    ensure
      STDERR.puts("exit #{join_point.signature}")
    end
  end
end

jobs = [PrintJob.new, BackupJob.new]
jobs.each(&:run)
